;**********************************************************************
;                                                                     *
;    Filename:	    mtx-015.asm                                       *
;    Date:                                                            *
;    File Version:                                                    *
;                                                                     *
;    Author:        el@jap.hu                                         *
;                   http://jap.hu/                                    *
;**********************************************************************
;    Notes:
;
;RF transmitter prot.v4 (Manchester)
;supposed crystal freq is 4MHz
;frequency is 400/800 Hz
; delay times = 625 usec, 625 usec
;**********************************************************************
;v4 HISTORY
;
;011-20010109
;manchester coded output
;
;012-20010113
;continous modulation out (output 1-bits while idle)
;one button only, planning 5 for the final version
;
; header : 8xbit1, 1xbit0
; bytes : 8xbitX, 1xbit1, 1xbit0
;
;013-20010113
;packet struct: +1 byte, new devid: 9ae6
;should send 5 button states
;
;015-20010124
;combined 013 and 010
;one-button version, transmit number of times the button is pressed
;transmit a signal five times
;
;**********************************************************************

;define F84
#define C508
;define TXINV
;invert TX output so inactive is HIGH?

#ifdef F84
	list      p=16F84             ; list directive to define processor
	#include <p16F84.inc>         ; processor specific variable definitions

	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC
	;WDT=watchdog, PWRTE=power on timer, XT= crystal osc

freemem EQU 0x10
txbit	EQU 1
ledbit	EQU 2
oscbit	EQU 4
ioport	EQU PORTB
#define BTN	ioport,4
#endif

#ifdef C508
	list	p=12c508
	#include <p12C508.inc>

	__CONFIG   _CP_OFF & _WDT_OFF & _MCLRE_OFF & _IntRC_OSC

freemem EQU 0x07
txbit	EQU 0x20
ledbit	EQU 0x04
oscbit	EQU 0x10
ioport	EQU GPIO
#define BTN	ioport,0
#endif

#ifndef TXINV
#define TXLOW 0
#define TXHIGH txbit
#else
#define TXLOW txbit
#define TXHIGH 0
#endif

;***** VARIABLE DEFINITIONS

count1		EQU	freemem
count2		EQU	freemem+1
count3		EQU	freemem+2
count4		EQU	freemem+3
ncnt		EQU	freemem+4 ; ndelay counter
bt		EQU	freemem+5
sum		EQU	freemem+6
buffer		EQU	freemem+7
bdevid		EQU	buffer  ; 2 byte DEVID
bdata		EQU	buffer+2; 'real' data

delayconst	EQU	.202 ; ~625 usec @4MHz, half_frame delay

;**********************************************************************
#ifdef F84
		ORG     0x000             ; processor reset vector
  		goto    main              ; go to beginning of program
		nop
		nop
		nop

		;ORG     0x004             ; interrupt vector location
		retfie
#endif

#ifdef C508
		ORG	0x000
		movwf	OSCCAL
		;C508 HARDWARE INIT
		clrf	GPIO ; clear all I/O pins
		movlw	3 ; GP0, GP1, GP3 input
		TRIS	GPIO ; I/O directions

		movlw	0
		OPTION
		;-GPWU wakeup ENA
		;-GPPU pullup ENA on GP0,1,3
		;T0CS disable: enablings implicitly sets GP2 dir as input!

		goto	main
#endif

table		addwf PCL, F
devid		dt 0xc1, 0x4e, 0, 0x5a

		; for the 12-bit devices
bit0		goto bit0_1
bit1		goto bit1_1
readbuf		goto readbuf_1
outbyte		goto outbyte_1
delay		goto delay_1
readbyte	goto readbyte_1

		FILL 0xfff, 0xee ; page 00-ff

main		
#ifdef F84
		;F84 HARDWARE INIT
		clrf PORTA
		clrf PORTB

		MOVLW 0xf8 ; button = PB4
		TRIS PORTB
		MOVLW 0; PA all out, not used
		TRIS PORTA

		movlw 0x3
		OPTION ; pres 1:16, B4-B7 PU ena
#endif
		call readbuf

loop		movlw TXLOW ; turn oscbit OFF
		movwf ioport
		; check if the button is pressed
		call delay
		btfss   BTN       ;wait for button press
		goto	keys_pressed

no_keys		;no keys pressed, going to sleep
		movf ioport, W

#ifdef F84
		movlw 8 ; RBIE portB change int.ena
		movwf INTCON ;and clear RBIF (bit#0)
#endif
		sleep   ; the C508 will reset on WAKE-UP!
			; the F84 will continue from here
		movf ioport, W
		goto loop

keys_pressed	; at least one key is pressed in this cycle

		movlw	ledbit+TXLOW ; show green while waiting
		movwf	ioport
		call	readbyte
		movlw	TXLOW+oscbit
		movwf	ioport
		call delay ; ~300us OSC turn-on time is needed for a SAW device
			   ; delay is around 30 ms

		movlw 0x0f
		andwf bt, W
		movwf bdata ; store 4-bit key data in buffer
		incf bdata+1, F ; the C508 will reset it before every press...

		movlw 5
		movwf count4 ; transmit repetition

warm		; output 11 header bits to quit false rxbuf
		movlw 0x20 ; 32xbit1
		movwf count2
w_head		call bit1
		decfsz count2,F
		goto w_head

outbuf		movlw 0x20 ; 32xbit1, 1xbit0

header		movwf count2
head0		call bit1
		decfsz count2,F
		goto head0
		call bit0

		movlw buffer
		movwf FSR
		movlw 0x04
		movwf count1
		clrf sum
		;
outbu0		movf INDF,W
		addwf sum,F
		movf INDF,W
		call outbyte
		incf FSR,F
		decfsz count1,F
		goto outbu0
		movf sum,W
		call outbyte
		; buffer is sent

		movlw 0x10
		addwf bdata, F ; increase upper 4 bits

		decfsz count4, F
		goto outbuf ; repeat buffer

		goto loop

		dw 0xfff
		;

outbyte_1	movwf bt
		movlw 8
		movwf count2
outby0		rlf bt,F
		btfsc STATUS,C
		goto outby1
		call bit0
		goto outby2
outby1		call bit1
outby2		decfsz count2,F
		goto outby0
		;
		call bit1
		; and bit0

bit0_1		movlw TXHIGH+oscbit ; HIGH
		movwf ioport

ndelaya0	movlw delayconst
		movwf ncnt
ndelaya1	decfsz ncnt, F
		goto ndelaya1

		movlw TXLOW+oscbit ; to LOW transition
		movwf ioport

ndelayb0	movlw delayconst
		movwf ncnt
ndelayb1	decfsz ncnt, F
		goto ndelayb1

		return
		dw 0xfff

bit1_1		movlw TXLOW+oscbit ; LOW
		movwf ioport

ndelayc0	movlw delayconst
		movwf ncnt
ndelayc1	decfsz ncnt, F
		goto ndelayc1

		movlw TXHIGH+oscbit ; to HIGH transition
		movwf ioport

ndelaye0	movlw delayconst
		movwf ncnt
ndelaye1	decfsz ncnt, F
		goto ndelaye1

		return
		dw 0xfff

readbuf_1	;read data into buffer
		movlw buffer
		movwf FSR
		clrf count1
readbu0		movf count1, W
		call table
		movwf INDF
		incf FSR,F
		incf count1,F

		movlw 0x04
		subwf count1, W
		bnz readbu0

		return
		dw 0xfff

readbyte_1	clrf	bt
read1		clrf    count1
read0		call    delay
		incf    count1, F
		movlw	.20             ;timeout
		subwf	count1, W
		bc      read_timeout
		btfsc   BTN       ;wait for button press
		goto    read0
		;button pressed

		incf    bt, F		;increase value
read2		call    delay
		btfss   BTN       ;wait for button release
		goto    read2

		goto    read1

read_timeout	; no button pressed for a time
		return
		dw 0xfff

delay_1
delay0		movlw .100
		movwf count2

delay1		movlw .99
		movwf count3

delay2		decfsz count3, F
		goto delay2

delay3		decfsz count2, F
		goto delay1
		
		return

		end

